#ifndef TABS_TABS_H
#define TABS_TABS_H

/*
 * autor
 * Michal Podsiadlik
 * michal at gov.one.pl
 */

#include <qobject.h>
#include <qtimer.h>

#include <misc.h>
#include <chat_manager.h>
#include <kadu.h>

#include "tabwidget.h"

class QPopupMenu;
class Action;

class TabsManager : public QObject
{
	Q_OBJECT

	public:
		TabsManager();
		~TabsManager();

		/**
		* Odcza rozmow od kart.
		* @param chat wskanik do rozmowy
		* @return prawda jeli rozmowa bya w kartach
		*/
		bool detachChat(Chat* chat);

	public slots:
		/**
		* Slot zostaje wywoany w momencie otwarcia okna rozmowy.
		* @param senders lista uytkownikw rozmowy
		*/
		void onNewChat(const UserGroup* senders);

		/**
		* Slot zostaje wywoany w momencie zamknicia rozmowy.
		* @param senders lista uytkownikw rozmowy
		*/
		void onDestroyingChat(const UserGroup* senders);

		/**
		* Slot zostaje wywoany za kadym razem gdy nastpi prba otwarcia okna rozmowy.
		* @param senders lista uytkownikw rozmowy
		*/
		void onOpenChat(UserListElements senders);

		/**
		* Slot zostaje wywoany w momencie zmiany statusu.
		* @param ule kontakt, dla ktrego zmieni si status
		* @param protocolName identyfikator protokou
		* @param oldStatus poprzedni status
		* @param massively czy jest to fragment wikszych zmian
		* @param last jeeli massively == true, to last == true dla ostatniego kontaktu
		*/
		void onStatusChanged(UserListElement ule, QString protocolName, const UserStatus &, bool, bool);

		/**
		* Slot zostaje wywoany w momencie otrzymania wiadomoci.
		* @param protocol protok dostarczajcy wiadomo
		* @param senders lista uytkownikw rozmowy
		* @param msg tre wiadomoci
		* @param time czas
		* @param grab wiadomo zostaa przejta (tak/nie)
		*/
		void onChatMsgReceived(Protocol* protocol, UserListElements senders, const QString& msg, time_t time, bool& grab);

		void onTabChange(QWidget *);

		/**
		* Slot zostaje wywoany w momencie zatwierdzenia zmian w konfiguracji.
		*/
		void onApplyConfig();
		void onNewTab();
		void onPopupMenu();

		/**
		* Slot zostaje wywoany w momencie zmiany stanu przycisku doczenia/odczania okna do/od kart.
		* @param users lista uytkownikw rozmowy
		* @param source wskanik do widgeta
		* @param isOn stan przycisku
		*/
		void onTabAttach(const UserGroup* users, const QWidget* source, bool isOn);

		/**
		* Slot zostaje wywoany w momencie zamknicia karty.
		* Zamyka rozmow.
		* @param w wkanik do obiektu okna ktre bdziemy zamyka
		*/
		void onDeleteTab(QWidget* w);

		/**
		* Slot zostaje wywoany w momencie zmiany w konfiguracji opcji "Use tabs by default".
		* Aktywuje lub deaktywuje w konfiguracji opcj "Minimum number of tabs".
		* @param toggled stan opcji (wczona/wyczona)
		*/
		void onToggledDefaultTabs(bool toggled);

		/**
		* Slot zostaje wywoany w momencie utworzenia okna konfiguracji.
		*/
		void onCreateConfigDialog();

		/**
		* Slot zostaje wywoany w momencie dodania przycisku do toolbara.
		* @param users lista uytkownikw rozmowy
		*/
		void onAddedToToolbar(const UserGroup* users);
	private:
		TabWidget* tabdialog;
		QTimer timer;
		QValueList<Chat *> chatsWithNewMessages, newchats;
		bool no_tabs, autoswith;
		void insertTab(Chat* chat);
		void makePopupMenu();
		int menuitem;
		Chat* selectedchat;
		QPopupMenu* menu;
		Action* action;

		/**
		* Przywraca stan kart w momencie uruchomienia kadu.
		*/
		void loadTabs();

		/**
		* Zapisuje stan kart w momencie wyjcia z kadu.
		*/
		void saveTabs();

	private slots:
		void onTimer();
		void onContextMenu(QWidget* w, const QPoint& pos);
		void onMenu(int id);
};

extern TabsManager* tabs_manager;
#endif
